@@ -12,7 +12,7 @@ import operator
 import time
 from warnings import warn
 from datetime import datetime
-from zipfile import ZipFile, ZipInfo, ZIP_DEFLATED
+from zipfile import ZipFile, ZipInfo, ZIP_DEFLATED, LargeZipFile
 from struct import unpack
 
 from .compatibility import int_types, num_types, str_types, force_unicode
@@ -36,8 +36,11 @@ from .chart_scatter import ChartScatter
 from .chart_stock import ChartStock
 from .exceptions import InvalidWorksheetName
 from .exceptions import DuplicateWorksheetName
+from .exceptions import ReservedWorksheetName
 from .exceptions import UndefinedImageSize
 from .exceptions import UnsupportedImageFormat
+from .exceptions import FileCreateError
+from .exceptions import FileSizeError
 
 
 class Workbook(xmlwriter.XMLwriter):
@@ -285,6 +288,9 @@ class Workbook(xmlwriter.XMLwriter):
                  % force_unicode(vba_project))
             return -1
 
+        if self.vba_codename is None:
+            self.vba_codename = 'ThisWorkbook'
+
         self.vba_project = vba_project
         self.vba_is_stream = is_stream
 
@@ -300,8 +306,16 @@ class Workbook(xmlwriter.XMLwriter):
 
         """
         if not self.fileclosed:
-            self.fileclosed = 1
-            self._store_workbook()
+
+            try:
+                self._store_workbook()
+            except IOError as e:
+                raise FileCreateError(e)
+            except LargeZipFile as e:
+                raise FileSizeError("Filesize would require ZIP64 extensions. "
+                                    "Use workbook.use_zip64().")
+
+            self.fileclosed = True
 
     def set_size(self, width, height):
         """
@@ -604,6 +618,14 @@ class Workbook(xmlwriter.XMLwriter):
         self._xml_close()
 
     def _store_workbook(self):
+
+        # Create the xlsx/zip file.
+        try:
+            xlsx_file = ZipFile(self.filename, "w", compression=ZIP_DEFLATED,
+                                allowZip64=self.allow_zip64)
+        except IOError as e:
+            raise e
+
         # Assemble worksheets into a workbook.
         packager = Packager()
 
@@ -621,6 +643,12 @@ class Workbook(xmlwriter.XMLwriter):
             if sheet.index == self.worksheet_meta.activesheet:
                 sheet.active = 1
 
+        # Set the sheet vba_codename the workbook has a vbaProject binary.
+        if self.vba_project:
+            for sheet in self.worksheets():
+                if sheet.vba_codename is None:
+                    sheet.set_vba_name()
+
         # Convert the SST strings data structure.
         self._prepare_sst_string_data()
 
@@ -648,11 +676,10 @@ class Workbook(xmlwriter.XMLwriter):
         # Free up the Packager object.
         packager = None
 
-        xlsx_file = ZipFile(self.filename, "w", compression=ZIP_DEFLATED,
-                            allowZip64=self.allow_zip64)
-
         # Add XML sub-files to the Zip file with their Excel filename.
-        for os_filename, xml_filename, is_binary in xml_files:
+        for file_id, file_data in enumerate(xml_files):
+            os_filename, xml_filename, is_binary = file_data
+
             if self.in_memory:
 
                 # Set sub-file timestamp to Excel's timestamp of 1/1/1980.
@@ -667,14 +694,21 @@ class Workbook(xmlwriter.XMLwriter):
                     xlsx_file.writestr(zipinfo,
                                        os_filename.getvalue().encode('utf-8'))
             else:
-                # The files are tempfiles.
+                # The sub-files are tempfiles on disk, i.e, not in memory.
 
-                # Set sub-file timestamp to Excel's timestamp of 1/1/1980.
-                timestamp = time.mktime((1980, 1, 1, 0, 0, 0, 0, 0, -1))
+                # Set sub-file timestamp to 31/1/1980 due to portability
+                # issues setting it to Excel's timestamp of 1/1/1980.
+                timestamp = time.mktime((1980, 1, 31, 0, 0, 0, 0, 0, -1))
                 os.utime(os_filename, (timestamp, timestamp))
 
-                xlsx_file.write(os_filename, xml_filename)
-                os.remove(os_filename)
+                try:
+                    xlsx_file.write(os_filename, xml_filename)
+                    os.remove(os_filename)
+                except LargeZipFile as e:
+                    # Close open temp files on zipfile.LargeZipFile exception.
+                    for i in range(file_id, len(xml_files)-1):
+                        os.remove(xml_files[i][0])
+                    raise e
 
         xlsx_file.close()
 
@@ -739,12 +773,23 @@ class Workbook(xmlwriter.XMLwriter):
                 "Excel worksheet name '%s' must be <= 31 chars." %
                 sheetname)
 
-        # Check that sheetname doesn't contain any invalid characters
+        # Check that sheetname doesn't contain any invalid characters.
         if invalid_char.search(sheetname):
             raise InvalidWorksheetName(
                 "Invalid Excel character '[]:*?/\\' in sheetname '%s'." %
                 sheetname)
 
+        # Check that sheetname doesn't start or end with an apostrophe.
+        if sheetname.startswith("'") or sheetname.endswith("'"):
+            raise InvalidWorksheetName(
+                "Sheet name cannot start or end with an apostrophe \"%s\"." %
+                sheetname)
+
+        # Check that sheetname isn't the reserved work "History".
+        if sheetname.lower() == 'history':
+            raise ReservedWorksheetName(
+                "Worksheet name 'History' is reserved by Excel")
+
         # Check that the worksheet name doesn't already exist since this is a
         # fatal Excel error. The check must be case insensitive like Excel.
         for worksheet in self.worksheets():
@@ -1416,7 +1461,6 @@ class Workbook(xmlwriter.XMLwriter):
         vml_shape_id = 1024
         vml_files = 0
         comment_files = 0
-        has_button = False
 
         for sheet in self.worksheets():
             if not sheet.has_vml and not sheet.has_header_vml:
@@ -1449,25 +1493,12 @@ class Workbook(xmlwriter.XMLwriter):
             self.num_vml_files = vml_files
             self.num_comment_files = comment_files
 
-            if len(sheet.buttons_list):
-                has_button = True
-
-                # Set the sheet vba_codename if it has a button and the
-                # workbook has a vbaProject binary.
-                if self.vba_project and sheet.vba_codename is None:
-                    sheet.set_vba_name()
-
         # Add a font format for cell comments.
         if comment_files > 0:
             xf = self.add_format({'font_name': 'Tahoma', 'font_size': 8,
                                   'color_indexed': 81, 'font_only': True})
             xf._get_xf_index()
 
-        # Set the workbook vba_codename if one of the sheets has a button and
-        # the workbook has a vbaProject binary.
-        if has_button and self.vba_project and self.vba_codename is None:
-            self.set_vba_name()
-
     def _prepare_tables(self):
         # Set the table ids for the worksheet tables.
         table_id = 0
